//This Wall Follower averages the last 5 readings from the IR Pin, creating a smoother Run.
// The Motor Controller is a Pololu Dual Serial Low Voltage.

unsigned char Mbuff[4];   // Declare the Motor Control Char array
const int IRPin = 1;      // Sharp 4cm = 30cm IR Range Finder
const int motor_reset = 2;

int LastCMD = 1;

int IRAverage[5];   
int ThisRead = 0;
int Total = 0;
int Distance = 0;

void setup()
{
  pinMode(motor_reset, OUTPUT);
  Serial.begin(9600);
  
  for (ThisRead = 0; ThisRead < 5; ThisRead++)   // Set's all IR Readings to 0
  {
    IRAverage[ThisRead] = 0;
  }
  ThisRead = 0;
  
    Mbuff[0]=0x80;  // Assign the start byte to control the Pololu Low Voltage motor controller
    Mbuff[1]=0x00;  // Assign the Address of the Motor Controller
    Mbuff[2]=0x00;     // This is the Motor Number & Direction. ONLY ONE THAT CHANGES
    Mbuff[3]=0x45;  // Assign the Speed 0x00 (0) Stopped to 0x7F (179) Full Speed, 0x45 = 69 of a 179 total

  digitalWrite(motor_reset, LOW);    // Reset the Motor Controller
  delay(100);
  digitalWrite(motor_reset, HIGH);
  delay(200);
}

void loop()
{
   
   RollDistance();   // A Function that add's an average of the last 5 IR readings

   if (Distance > 330 && LastCMD != 4) // Really close to the wall
   {
     LastCMD = 4;    // This makes sure that the code is only sent to the Motor Controller when it changes and not every loop
     Mbuff[2]=0x02; //RIGHT MOTOR
     Mbuff[3]=0x08; // SPEED
     MWrite();  
   }
   else if (Distance > 250 && Distance < 330 && LastCMD !=3) // moving towards wall
   {
     LastCMD = 3;
     //speed up left a little
     //slow Right a little = turn Right
     Mbuff[2]=0x00; //LEFT MOTOR
     Mbuff[3]=0x49; // SPEED
     MWrite();
     Mbuff[2]=0x02; //RIGHT MOTOR
     Mbuff[3]=0x41; // SPEED
     MWrite();   
   }
   else if (Distance < 170 && LastCMD !=2)  // Moving Away from wall
   {
     LastCMD = 2;
     //slow left a little
     //speed Right a little = turn Left
     Mbuff[2]=0x00; //LEFT MOTOR
     Mbuff[3]=0x35; // SPEED
     MWrite();
     Mbuff[2]=0x02; //RIGHT MOTOR
     Mbuff[3]=0x4F; // SPEED
     MWrite();   
   }
   else if (Distance > 170 && Distance < 250 && LastCMD !=1)
   {
     LastCMD = 1;
     //set motors back to starting speeds, turning towards wall
     Mbuff[2]=0x00; //LEFT MOTOR
     Mbuff[3]=0x41; // SPEED
     MWrite();
     Mbuff[2]=0x02; //RIGHT MOTOR
     Mbuff[3]=0x49; // SPEED
     MWrite();   
   }
   delay(55); 
}



void RollDistance()    // The Rolling Averge Calculator
{
  Total = Total - IRAverage[ThisRead];   // Minus the oldest read
  IRAverage[ThisRead] = analogRead(IRPin);  // Replaces the oldest Read
  Total = Total + IRAverage[ThisRead];   // Adds the New Read to the total
  ThisRead ++;     // Increment the Reading number
  
  if (ThisRead >= 5)   // Loops the Reading Number
  {
    ThisRead = 0;
  }
  Distance = Total / 5;
}

void MWrite()    //Writes the char's to the Tx
{
  for (int i = 0; i < 4; i++)
  {
    Serial.write(Mbuff, 4);
  }
}

   
